
#include <string.h>
#include "buffers.h"

void fill( buffer_t buffer, char fill_character )
{
    /*@ loop invariant
      @   \forall int j; 0 <= j < i ==> buffer[j] == fill_character;
      @ loop assigns buffer[0 .. BUFFER_SIZE - 1];
      @*/
    for( int i = 0; i < BUFFER_SIZE; ++i ) {
        buffer[i] = fill_character;
    }
}


int compact( buffer_t buffer, char erase_character, char fill_character )
{
    buffer_t workspace;
    int      offset = 0;
    int      valid;

    memset( workspace, ' ', BUFFER_SIZE );

    // Take interesting characters from the original buffer and pack them into workspace.
    /*@ loop invariant
      @   \forall int j; 0 <= j < offset ==> workspace[j] != erase_character;
      @ loop assigns buffer[0 .. BUFFER_SIZE - 1];
      @*/
    for( int i = 0; i < BUFFER_SIZE; ++i ) {
        if( buffer[i] != erase_character ) {
            workspace[offset] = buffer[i];
            offset++;
        }
    }

    valid = offset;

    // Install appropriate number of fill characters.
    /*@ loop invariant
      @   \forall int j; offset <= j < i ==> workspace[j] == fill_character;
      @ loop invariant
      @   \forall int j; 0 <= j < offset ==> workspace[j] != erase_character;
      @ loop assigns buffer[offset .. BUFFER_SIZE - 1];
      @*/
    for( int i = offset; i < BUFFER_SIZE; ++i ) {
        workspace[i] = fill_character;
    }

    memcpy( buffer, workspace, BUFFER_SIZE );
    return valid;
}


int count_and_erase( buffer_t buffer, char ch )
{
    int result = 0;

    // I would like to use a ghost variable here to hold a copy of the original buffer so I can
    // talk about the original buffer contents in the loop invariant. See the SPARK version.
    // Ghost code is only partially implemented in Frama-C right now.

    /*@ loop invariant
      @   0 <= result <= BUFFER_SIZE;
      @ loop invariant
      @   \forall int j; 0 <= j < i ==>
      @       buffer[i] != ch;
      @ loop assigns buffer[0 .. BUFFER_SIZE - 1];
      @*/
    for( int i = 0; i < BUFFER_SIZE; ++i ) {
        if( buffer[i] == ch ) {
            buffer[i] = ' ';
            result++;
        }
    }
    return result;
}
