// This is a buffers package following the Buffers package presented in SPARK.
// It is interesting to compare and contrast the two systems.

#ifndef BUFFERS_H
#define BUFFERS_H

#define BUFFER_SIZE 1024

typedef char buffer_t[BUFFER_SIZE];

/*@ requires \valid(buffer + (0 .. BUFFER_SIZE - 1));
  @ ensures \forall int i; 0 <= i < BUFFER_SIZE ==> buffer[i] == fill_character;
  @ assigns buffer[0 .. BUFFER_SIZE - 1];
  @*/
void fill( buffer_t buffer, char fill_character );

/*@ requires \valid(buffer + (0 .. BUFFER_SIZE - 1));
  @ ensures \forall int i; \result <= i < BUFFER_SIZE ==> buffer[i] == fill_character;
  @ ensures \forall int i; 0 <= i < \result ==> buffer[i] != erase_character;
  @ assigns  buffer[0 .. BUFFER_SIZE - 1];
  @*/
int compact( buffer_t buffer, char erase_character, char fill_character );

/*@ requires \valid(buffer + (0 .. BUFFER_SIZE - 1));
  @ ensures  0 <= \result <= BUFFER_SIZE;
  @ ensures  \forall int i; 0 <= i < BUFFER_SIZE ==>
  @              \old(buffer[i]) == ch ? buffer[i] == ' ' : buffer[i] == \old(buffer[i]);
  @ assigns buffer[0 .. BUFFER_SIZE - 1];
  @*/
int count_and_erase( buffer_t buffer, char ch );

#endif
